"""
All of the dialogs used throughout Gooey
"""
from collections import namedtuple

import wx

from gooey.gui.lang.i18n import _


# These don't seem to be specified anywhere in WX for some reason
DialogConstants = namedtuple('DialogConstants', 'YES NO')(5103, 5104)


def showDialog(title, content, style):
    dlg = wx.MessageDialog(None, content, title, style)
    dlg.SetYesNoLabels(_('dialog_button_yes'), _('dialog_button_no'))
    dlg.SetOKLabel(_('dialog_button_ok'))
    result = dlg.ShowModal()
    dlg.Destroy()
    return result


def missingArgsDialog():
    showDialog(_('error_title'), _('error_required_fields'), wx.ICON_ERROR)


def validationFailure():
    showDialog(_('error_title'), _('validation_failed'), wx.ICON_WARNING)


def showSuccess():
    showDialog(_('execution_finished'), _('success_message'), wx.ICON_INFORMATION)


def showFailure():
    showDialog(_('execution_finished'), _('uh_oh'), wx.ICON_ERROR)


def confirmExit():
    result = showDialog(_('sure_you_want_to_exit'), _('close_program'), wx.YES_NO | wx.ICON_INFORMATION)
    return result == DialogConstants.YES


def confirmForceStop():
    result = showDialog(_('stop_task'), _('sure_you_want_to_stop'), wx.YES_NO | wx.ICON_WARNING)
    return result == DialogConstants.YES




import unittest

from gooey.gui.components.options import options

class TestPrefixFilter(unittest.TestCase):

    def test_doc_schenanigans(self):
        """Sanity check that my docstring wrappers all behave as expected"""
        @options._include_layout_docs
        def no_self_docstring():
            pass

        @options._include_layout_docs
        def yes_self_docstring():
            """sup"""
            pass

        # gets attached to functions even if they don't have a docstring
        self.assertIn(options.LayoutOptions.__doc__, no_self_docstring.__doc__)
        # gets attached to the *end* of existing doc strings
        self.assertTrue(yes_self_docstring.__doc__.startswith('sup'))
        self.assertIn(options.LayoutOptions.__doc__, yes_self_docstring.__doc__)


    def test_clean_method(self):
        """
        _clean should drop any keys with None values
        and flatten the layout_option kwargs to the root level
        """
        result = options._clean({'a': None, 'b': 123, 'c': 0})
        self.assertEqual(result, {'b': 123, 'c': 0})

        result = options._clean({'root_level': 123, 'layout_options': {
            'nested': 'hello',
            'another': 1234
        }})
        self.assertEqual(result, {'root_level': 123, 'nested': 'hello', 'another': 1234})

    def test_only_provided_arguments_included(self):
        """
        More sanity checking that the internal use of locals()
        does the Right Thing
        """
        option = options.LayoutOptions(label_color='#ffffff')
        self.assertIn('label_color', option)

        option = options.LayoutOptions()
        self.assertNotIn('label_color', option)

        option = options.TextField(label_color='#ffffff')
        self.assertIn('label_color', option)

        option = options.TextField()
        self.assertNotIn('label_color', option)


import wx

from gooey.gui import formatters
from gooey.gui.components.widgets.bases import TextContainer


class Slider(TextContainer):
    """
    An integer input field
    """
    widget_class = wx.Slider
    def getWidget(self, *args, **options):
        widget = self.widget_class(self,
                                   minValue=self._options.get('min', 0),
                                   maxValue=self._options.get('max', 100),
                                   style=wx.SL_MIN_MAX_LABELS | wx.SL_VALUE_LABEL)
        return widget

    def getWidgetValue(self):
        return self.widget.GetValue()

    def setValue(self, value):
        self.widget.SetValue(value)

    def formatOutput(self, metatdata, value):
        return formatters.general(metatdata, str(value))





import wx
from collections import defaultdict

__ALL__ = ['pub']


class PubSub(object):
    """
    A super simplified clone of Wx.lib.pubsub since it doesn't exist on linux
    """

    def __init__(self):
        self.registry = defaultdict(list)

    def subscribe(self, event, handler):
        self.registry[event].append(handler)

    def send_message(self, event, **kwargs):
        for event_handler in self.registry.get(event, []):
            wx.CallAfter(event_handler, **kwargs)

    def send_message_sync(self, event, **kwargs):
        """
        ===== THIS IS NOT THREAD SAFE =====
        Synchronously sends the message to all relevant consumers
        and blocks until a response is received.

        This MUST ONLY be used for communication within
        the same thread! It exists primarily as an escape
        hatch for bubbling up messages (which would be garbage
        collected in the CallAfter form) to interested components
        """
        for event_handler in self.registry.get(event, []):
            event_handler(**kwargs)

pub = PubSub()



from gooey import Gooey
from gooey import GooeyParser


@Gooey(
    sidebar_title="Your Custom Title",
    show_sidebar=True,
    dump_build_config=True,
    show_success_modal=False,
    force_stop_is_error=False,
    language='chinese'
)
def main():
    desc = "Example application to show Gooey's various widgets"
    parser = GooeyParser(description=desc, add_help=False)

    parser.add_argument('--textfield', default=2, widget="TextField", gooey_options={
        'validator': {
            'test': 'int(user_input) > 5',
            'message': 'number must be greater than 5'
        }
    })
    parser.add_argument('--textarea', default="oneline twoline", widget='Textarea')
    parser.add_argument('--password', default="hunter42", widget='PasswordField')
    parser.add_argument('--commandfield', default="cmdr", widget='CommandField')
    parser.add_argument('--dropdown',
                        choices=["one", "two"], default="two", widget='Dropdown')
    parser.add_argument('--listboxie',
                    nargs='+',
                    default=['Option three', 'Option four'],
                    choices=['Option one', 'Option two', 'Option three',
                             'Option four'],
                    widget='Listbox',
                    gooey_options={
                        'height': 300,
                        'validate': '',
                        'heading_color': '',
                        'text_color': '',
                        'hide_heading': True,
                        'hide_text': True,
                    }
                )
    parser.add_argument('-c', '--counter', default=3, action='count', widget='Counter')
    #
    parser.add_argument("-o", "--overwrite", action="store_true",
                                default=True,
                                widget='CheckBox')

    ### Mutex Group ###
    verbosity = parser.add_mutually_exclusive_group(
        required=True,
        gooey_options={
            'initial_selection': 1
        }
    )
    verbosity.add_argument(
        '--mutexone',
        default=True,
        action='store_true',
        help="Show more details")

    verbosity.add_argument(
        '--mutextwo',
        default='mut-2',
        widget='TextField')

    parser.add_argument("--filechooser", default="fc-value", widget='FileChooser')
    parser.add_argument("--filesaver", default="fs-value", widget='FileSaver')
    parser.add_argument("--dirchooser", default="dc-value", widget='DirChooser')
    parser.add_argument("--datechooser", default="2015-01-01", widget='DateChooser')
    parser.add_argument("--colourchooser", default="#000000", widget='ColourChooser')

    dest_vars = [
        'textfield',
        'textarea',
        'password',
        'commandfield',
        'dropdown',
        'listboxie',
        'counter',
        'overwrite',
        'mutextwo',
        'filechooser',
        'filesaver',
        'dirchooser',
        'datechooser',
        'colourchooser'

    ]


    args = parser.parse_args()
    import time
    time.sleep(3)
    for i in dest_vars:
        assert getattr(args, i) is not None
    print("Success")


if __name__ == '__main__':
    main()



import unittest
from argparse import ArgumentParser
from itertools import *

from tests.harness import instrumentGooey
from gooey.tests import *

class TestGooeyHeader(unittest.TestCase):

    def make_parser(self):
        parser = ArgumentParser(description='description')
        return parser

    def test_header_visibility(self):
        """
        Test that the title and subtitle components correctly show/hide
        based on config settings.

        Verifying Issue #497
        """
        for testdata in self.testcases():
            with self.subTest(testdata):
                with instrumentGooey(self.make_parser(), **testdata) as (app, gooeyApp):
                    header = gooeyApp.header

                    self.assertEqual(
                        header._header.IsShown(),
                        testdata.get('header_show_title', True)
                    )

                    self.assertEqual(
                        header._subheader.IsShown(),
                        testdata.get('header_show_subtitle', True)
                    )


    def test_header_string(self):
        """
        Verify that string in the buildspec get correctly
        placed into the UI.
        """
        parser = ArgumentParser(description='Foobar')
        with instrumentGooey(parser, program_name='BaZzEr') as (app, gooeyApp):
            self.assertEqual(gooeyApp.header._header.GetLabelText(), 'BaZzEr')
            self.assertEqual(gooeyApp.header._subheader.GetLabelText(), 'Foobar')


    def testcases(self):
        """
        Generate a powerset of all possible combinations of
        the header parameters (empty, some present, all present, all combos)
        """
        iterable = product(['header_show_title', 'header_show_subtitle'], [True, False])
        allCombinations = list(powerset(iterable))
        return [{k: v for k,v in args}
                for args in allCombinations]


def powerset(iterable):
    "powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)"
    s = list(iterable)
    return chain.from_iterable(combinations(s, r) for r in range(len(s)+1))


if __name__ == '__main__':
    unittest.main()


"""
Primary orchestration and control point for Gooey.
"""

import sys

import wx
from wx.adv import TaskBarIcon

from gooey.gui import cli
from gooey.gui import events
from gooey.gui import seeder
from gooey.gui.components import modals
from gooey.gui.components.config import ConfigPage, TabbedConfigPage
from gooey.gui.components.console import Console
from gooey.gui.components.footer import Footer
from gooey.gui.components.header import FrameHeader
from gooey.gui.components.menubar import MenuBar
from gooey.gui.components.sidebar import Sidebar
from gooey.gui.components.tabbar import Tabbar
from gooey.gui.components.submit import Submit
from gooey.gui.lang.i18n import _
from gooey.gui.processor import ProcessController
from gooey.gui.util.time import Timing
from gooey.gui.pubsub import pub
from gooey.gui.util import wx_util
from gooey.gui.util.wx_util import transactUI
from gooey.python_bindings import constants


class GooeyApplication(wx.Frame):
    """
    Main window for Gooey.
    """

    def __init__(self, buildSpec, *args, **kwargs):
        super(GooeyApplication, self).__init__(None, *args, **kwargs)
        self._state = {}
        self.buildSpec = buildSpec

        self.applyConfiguration()
        self.menu = MenuBar(buildSpec)
        self.SetMenuBar(self.menu)
        self.header = FrameHeader(self, buildSpec)
        self.configs = self.buildConfigPanels(self)
        self.navbar = self.buildNavigation()
        self.footer = Footer(self, buildSpec)
        self.console = Console(self, buildSpec)
        self.layoutComponent()
        self.timer = Timing(self)

        self.clientRunner = ProcessController(
            self.buildSpec.get('progress_regex'),
            self.buildSpec.get('progress_expr'),
            self.buildSpec.get('hide_progress_msg'),
            self.buildSpec.get('encoding'),
            self.buildSpec.get('requires_shell'),
        )

        pub.subscribe(events.WINDOW_START, self.onStart)
        pub.subscribe(events.WINDOW_RESTART, self.onStart)
        pub.subscribe(events.WINDOW_STOP, self.onStopExecution)
        pub.subscribe(events.WINDOW_CLOSE, self.onClose)
        pub.subscribe(events.WINDOW_CMD, self.onPrintCMD)
        pub.subscribe(events.WINDOW_SUBMIT, self.onSubmit)
        pub.subscribe(events.WINDOW_CANCEL, self.onCancel)
        pub.subscribe(events.WINDOW_EDIT, self.onEdit)
        pub.subscribe(events.CONSOLE_UPDATE, self.console.logOutput)
        pub.subscribe(events.EXECUTION_COMPLETE, self.onComplete)
        pub.subscribe(events.PROGRESS_UPDATE, self.footer.updateProgressBar)
        pub.subscribe(events.TIME_UPDATE, self.footer.updateTimeRemaining)
        # Top level wx close event
        self.Bind(wx.EVT_CLOSE, self.onClose)

        if self.buildSpec['poll_external_updates']:
            self.fetchExternalUpdates()

        if self.buildSpec.get('auto_start', False):
            self.onStart()

    def applyConfiguration(self):
        self.SetTitle(self.buildSpec['program_name'])
        self.SetBackgroundColour(self.buildSpec.get('body_bg_color'))

    def onSubmit(self, *args, **kwargs):
        with transactUI(self):
            config = self.navbar.getActiveConfig()
            config.resetErrors()
            if config.isValid():
                sub = Submit(self)
                sub.ShowModal()
                if not sub.cancled:
                    self.showConsole()
                    cmd = self.buildCliString()
                    replacment_dict = {
                        "XXX_SXCMD_LINE_XXX": cmd,
                        "XXX_SXMPI_NPROC_XXX": str(1),
                        "XXX_SXMPI_JOB_NAME_XXX": "crYOLO",


                    }
                    new_script_pth = sub.generate_submission_script(path_template=sub.submission_template, replacement_dict=replacment_dict)
                    submit_command = " ".join([sub.submission_cmd, new_script_pth])
                    self.console.appendText(submit_command)
                    import subprocess
                    import shlex
                    process = subprocess.Popen(shlex.split(submit_command),
                                               stdout=subprocess.PIPE,
                                               stderr=subprocess.PIPE)
                    #stdout, stderr = process.communicate()
                    if self.buildSpec.get('return_to_config', False):
                        self.showSettings()
                    else:
                        self.showSuccess()
                        if self.buildSpec.get('show_success_modal', True):
                            wx.CallAfter(modals.showSuccess)

            else:
                config.displayErrors()
                self.Layout()

    def onPrintCMD(self, *args, **kwarg):

        with transactUI(self):
            config = self.navbar.getActiveConfig()
            config.resetErrors()
            if config.isValid():
                self.showConsole()
                self.console.appendText(self.buildCliString())
                if self.buildSpec.get('return_to_config', False):
                    self.showSettings()
                else:
                    self.showSuccess()
                    if self.buildSpec.get('show_success_modal', True):
                        wx.CallAfter(modals.showSuccess)
            else:
                config.displayErrors()
                self.Layout()

    def onStart(self, *args, **kwarg):
        """
        Verify user input and kick off the client's program if valid
        """
        with transactUI(self):
            config = self.navbar.getActiveConfig()
            config.resetErrors()
            if config.isValid():
                if self.buildSpec['clear_before_run']:
                    self.console.clear()
                self.clientRunner.run(self.buildCliString())
                self.showConsole()
            else:
                config.displayErrors()
                self.Layout()
        
    def onEdit(self):
        """Return the user to the settings screen for further editing"""
        with transactUI(self):
            if self.buildSpec['poll_external_updates']:
                self.fetchExternalUpdates()
            self.showSettings()


    def buildCliString(self):
        """
        Collect all of the required information from the config screen and
        build a CLI string which can be used to invoke the client program
        """
        config = self.navbar.getActiveConfig()
        group = self.buildSpec['widgets'][self.navbar.getSelectedGroup()]
        positional = config.getPositionalArgs()
        optional = config.getOptionalArgs()
        return cli.buildCliString(
            self.buildSpec['target'],
            group['command'],
            positional,
            optional,
            suppress_gooey_flag=self.buildSpec['suppress_gooey_flag']
        )


    def onComplete(self, *args, **kwargs):
        """
        Display the appropriate screen based on the success/fail of the
        host program
        """
        with transactUI(self):
            if self.clientRunner.was_success():
                if self.buildSpec.get('return_to_config', False):
                    self.showSettings()
                else:
                    self.showSuccess()
                    if self.buildSpec.get('show_success_modal', True):
                        wx.CallAfter(modals.showSuccess)
            else:
                if self.clientRunner.wasForcefullyStopped:
                    self.showForceStopped()
                else:
                    self.showError()
                    if self.buildSpec.get('show_failure_modal'):
                        wx.CallAfter(modals.showFailure)

    def onCancel(self):
        """Close the program after confirming

        We treat the behavior of the "cancel" button slightly
        differently than the general window close X button only
        because this is 'part of' the form.
        """
        if modals.confirmExit():
            self.onClose()


    def onStopExecution(self):
        """Displays a scary message and then force-quits the executing
        client code if the user accepts"""
        if self.shouldStopExecution():
            self.clientRunner.stop()


    def onClose(self, *args, **kwargs):
        """Stop any actively running client program, cleanup the top
        level WxFrame and shutdown the current process"""
        # issue #592 - we need to run the same onStopExecution machinery
        # when the exit button is clicked to ensure everything is cleaned
        # up correctly.
        if self.clientRunner.running():
            if self.shouldStopExecution():
                self.clientRunner.stop()
                self.destroyGooey()
        else:
            self.destroyGooey()


    def shouldStopExecution(self):
        return not self.buildSpec['show_stop_warning'] or modals.confirmForceStop()


    def destroyGooey(self):
        self.Destroy()
        sys.exit()

    def fetchExternalUpdates(self):
        """
        !Experimental!
        Calls out to the client code requesting seed values to use in the UI
        !Experimental!
        """
        seeds = seeder.fetchDynamicProperties(
            self.buildSpec['target'],
            self.buildSpec['encoding']
        )
        for config in self.configs:
            config.seedUI(seeds)

    def layoutComponent(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.header, 0, wx.EXPAND)
        sizer.Add(wx_util.horizontal_rule(self), 0, wx.EXPAND)

        sizer.Add(self.navbar, 1, wx.EXPAND)
        sizer.Add(self.console, 1, wx.EXPAND)
        sizer.Add(wx_util.horizontal_rule(self), 0, wx.EXPAND)
        sizer.Add(self.footer, 0, wx.EXPAND)
        self.SetMinSize((400, 300))
        self.SetSize(self.buildSpec['default_size'])
        self.SetSizer(sizer)
        self.console.Hide()
        self.Layout()
        if self.buildSpec.get('fullscreen', True):
            self.ShowFullScreen(True)
        # Program Icon (Windows)
        icon = wx.Icon(self.buildSpec['images']['programIcon'], wx.BITMAP_TYPE_PNG)
        self.SetIcon(icon)
        if sys.platform != 'win32':
            # OSX needs to have its taskbar icon explicitly set
            # bizarrely, wx requires the TaskBarIcon to be attached to the Frame
            # as instance data (self.). Otherwise, it will not render correctly.
            self.taskbarIcon = TaskBarIcon(iconType=wx.adv.TBI_DOCK)
            self.taskbarIcon.SetIcon(icon)



    def buildNavigation(self):
        """
        Chooses the appropriate layout navigation component based on user prefs
        """
        if self.buildSpec['navigation'] == constants.TABBED:
            navigation = Tabbar(self, self.buildSpec, self.configs)
        else:
            navigation = Sidebar(self, self.buildSpec, self.configs)
            if self.buildSpec['navigation'] == constants.HIDDEN:
                navigation.Hide()
        return navigation


    def buildConfigPanels(self, parent):
        page_class = TabbedConfigPage if self.buildSpec['tabbed_groups'] else ConfigPage

        return [page_class(parent, widgets, self.buildSpec)
                for widgets in self.buildSpec['widgets'].values()]


    def showSettings(self):
        self.navbar.Show(True)
        self.console.Show(False)
        self.header.setImage('settings_img')
        self.header.setTitle(_("settings_title"))
        self.header.setSubtitle(self.buildSpec['program_description'])
        self.footer.showButtons('cancel_button', 'start_button','print_cmd_button','submit_button')
        self.footer.progress_bar.Show(False)
        self.footer.time_remaining_text.Show(False)


    def showConsole(self):
        self.navbar.Show(False)
        self.console.Show(True)
        self.header.setImage('running_img')
        self.header.setTitle(_("running_title"))
        self.header.setSubtitle(_('running_msg'))
        self.footer.showButtons('stop_button')
        self.footer.progress_bar.Show(True)
        self.footer.time_remaining_text.Show(False)
        if self.buildSpec.get('timing_options')['show_time_remaining']:
            self.timer.start()
            self.footer.time_remaining_text.Show(True)
        if not self.buildSpec['progress_regex']:
            self.footer.progress_bar.Pulse()


    def showComplete(self):
        self.navbar.Show(False)
        self.console.Show(True)
        buttons = (['edit_button', 'restart_button', 'close_button']
                   if self.buildSpec.get('show_restart_button', True)
                   else ['edit_button', 'close_button'])
        self.footer.showButtons(*buttons)
        self.footer.progress_bar.Show(False)
        if self.buildSpec.get('timing_options')['show_time_remaining']:
            self.timer.stop()
        self.footer.time_remaining_text.Show(True)
        if self.buildSpec.get('timing_options')['hide_time_remaining_on_complete']:
            self.footer.time_remaining_text.Show(False)



    def showSuccess(self):
        self.showComplete()
        self.header.setImage('check_mark')
        self.header.setTitle(_('finished_title'))
        self.header.setSubtitle(_('finished_msg'))
        self.Layout()


    def showError(self):
        self.showComplete()
        self.header.setImage('error_symbol')
        self.header.setTitle(_('finished_title'))
        self.header.setSubtitle(_('finished_error'))


    def showForceStopped(self):
        self.showComplete()
        if self.buildSpec.get('force_stop_is_error', True):
            self.showError()
        else:
            self.showSuccess()
        self.header.setSubtitle(_('finished_forced_quit'))





from . chooser import Chooser, FileChooser, FileSaver, DirChooser, DateChooser, TimeChooser, MultiFileChooser, MultiDirChooser, ColourChooser
from . text_input import PasswordInput, MultilineTextInput, TextInput



from gooey.gui import formatters
from gooey.gui.components.widgets import core
from gooey.gui.components.widgets.bases import TextContainer, BaseChooser


__ALL__ = [
    'FileChooser',
    'MultiFileChooser',
    'FileSaver',
    'DirChooser',
    'MultiDirChooser',
    'DateChooser',
    'ColourChooser',
    'TimeChooser'
]


class FileChooser(BaseChooser):
    widget_class = core.FileChooser


class MultiFileChooser(BaseChooser):
    widget_class = core.MultiFileChooser

    def formatOutput(self, metatdata, value):
        return formatters.multiFileChooser(metatdata, value)


class FileSaver(BaseChooser):
    widget_class = core.FileSaver


class DirChooser(BaseChooser):
    widget_class = core.DirChooser


class MultiDirChooser(BaseChooser):
    widget_class = core.MultiDirChooser

    def formatOutput(self, metadata, value):
        return formatters.multiFileChooser(metadata, value)


class DateChooser(BaseChooser):
    widget_class = core.DateChooser


class ColourChooser(BaseChooser):
    widget_class = core.ColourChooser


class TimeChooser(BaseChooser):
    widget_class = core.TimeChooser



"""
Basic constraints to ensure GooeyParser is fed all the info it needs
for various widget classes.

TODO: this should all live in the build_config stage here where it is used
within the GooeyParser directly. As is, logic is fragmented across files. Some
assertions happen in argparse_to_json, while others happen in GooeyParser.

Whenever refactoring happens, these should be removed from GooeyParser.
"""
from textwrap import dedent

def is_required(action):
    return action.required

def is_hidden(options):
    return not options.get('visible', True)

def has_validator(options):
    return bool(options.get('validator'))

def has_default(action):
    return bool(action.default)

def assert_visibility_requirements(action, options):
    if action.required and is_hidden(options) \
            and not (has_validator(options) or has_default(action)):
        raise ValueError(dedent(
            '''
            When using Gooey's hidden field functionality, you must either '
      
              (a) provide a default value, or '
              (b) provide a custom validator'
              
            Without one of those, your users will be unable to advance past 
            the configuration screen as they cannot interact with your 
            hidden field, and the default validator requires something to 
            be present for fields marked as `required`.  
            '''
        ))

def assert_listbox_constraints(widget, **kwargs):
    if widget and widget == 'Listbox':
        if not 'nargs' in kwargs or kwargs['nargs'] not in ['*', '+']:
            raise ValueError(
                'Gooey\'s Listbox widget requires that nargs be specified.\n'
                'Nargs must be set to either `*` or `+` (e.g. nargs="*")'
            )




'''
Created on Jan 15 2019

@author: Jonathan Schultz

This file contains code that allows the default argument values to be specified
on the command line.
'''

from argparse import _SubParsersAction

def parse_cmd_args(self, args=None):

  def prepare_to_read_cmd_args(item):
    '''
    Before reading the command-line arguments, we need to fudge a few things:
      1. If there are subparsers, we need a dest in order to know in which
         subparser the command-line values should be stored.
      2. Any required argument or mutex group needs to be made not required,
         otherwise it will be compulsory to enter those values on the command
         line.
    We save the everything as it was before the fudge, so we can restore later.
    '''
    for action in item._actions:
      if isinstance(action, _SubParsersAction):
        action.save_dest = action.dest
        if not action.dest:
          action.dest = '_subparser'
      else:
        action.save_required = action.required
        action.required = False
        action.save_nargs = action.nargs
        if action.nargs == '+':
          action.nargs = '*'
        elif action.nargs is None:
          action.nargs = '?'

    for mutex_group in item._mutually_exclusive_groups:
      mutex_group.save_required = mutex_group.required
      mutex_group.required = False

  def overwrite_default_values(item, cmd_args):
    '''
    Subsistute arguments provided on the command line in the place of the
    default values provided to argparse.
    '''
    for action in item._actions:
      if isinstance(action, _SubParsersAction):
        subparser_arg = getattr(cmd_args, action.dest, None)
        if subparser_arg:
          overwrite_default_values(action.choices[subparser_arg], cmd_args)
      else:
        dest = getattr(action, 'dest', None)
        if dest:
          cmd_arg = getattr(cmd_args, dest, None)
          if cmd_arg:
            action.default = cmd_arg

  def restore_original_configuration(item):
    '''
    Restore the old values as they were to start with.
    '''
    for action in item._actions:
      if isinstance(action, _SubParsersAction):
        action.dest = action.save_dest
        del action.save_dest
      else:
        action.required = action.save_required
        del action.save_required
        action.nargs = action.save_nargs
        del action.save_nargs

    for mutex_group in item._mutually_exclusive_groups:
      mutex_group.required = mutex_group.save_required
      del mutex_group.save_required

  prepare_to_read_cmd_args(self)
  overwrite_default_values(self, self.original_parse_args(args))
  restore_original_configuration(self)



from itertools import chain

from copy import deepcopy

from gooey.util.functional import compact


def buildCliString(target, cmd, positional, optional, suppress_gooey_flag=False):
    positionals = deepcopy(positional)
    if positionals:
        positionals.insert(0, "--")

    cmd_string = ' '.join(compact(chain(optional, positionals)))

    if cmd != '::gooey/default':
        cmd_string = u'{} {}'.format(cmd, cmd_string)

    # windows fixes
    import sys
    import os
    if sys.platform == "win32":
        target_splitted = target.split(' ')
        cryolo_cmd = target_splitted[2]
        cryolo_cmd_splitted = os.path.splitext(cryolo_cmd[1:-1])
        if cryolo_cmd_splitted[1] == '':
            cryolo_cmd = cryolo_cmd_splitted[0] + '.exe'
            target_splitted[2] = f"\"{cryolo_cmd}\""
            target = ' '.join(target_splitted)


    ignore_flag = '' if suppress_gooey_flag else '--ignore-gooey'
    return u'{} {} {}'.format(target, ignore_flag, cmd_string)



import unittest
from collections import namedtuple

from tests.harness import instrumentGooey
from gooey import GooeyParser
from gooey.tests import *

Case = namedtuple('Case', 'inputs initialExpected')


class TestCommonProperties(unittest.TestCase):
    """
    Test options and functionality
    common across all widgets.
    """

    def makeParser(self, **kwargs):
        parser = GooeyParser(description='description')
        parser.add_argument('--widget', **kwargs)
        return parser

    def testInitialValue(self):
        widgets = ['ColourChooser',
                   'CommandField',
                   'DateChooser', 'DirChooser', 'FileChooser', 'FileSaver',
                   'FilterableDropdown',  'MultiDirChooser', 'MultiFileChooser',
                   'PasswordField',  'TextField', 'Textarea', 'TimeChooser']

        cases = [
            # initial_value supersedes, default
            Case(
                {'default': 'default', 'gooey_options': {'initial_value': 'some val'}},
                'some val'),
            Case(
                {'gooey_options': {'initial_value': 'some val'}},
                 'some val'),
            Case(
                {'default': 'default', 'gooey_options': {}},
                 'default'),
            Case({'default': 'default'},
                 'default')
        ]

        for widgetName in widgets:
            with self.subTest(widgetName):
                for case in cases:
                    parser = self.makeParser(widget=widgetName, **case.inputs)
                    with instrumentGooey(parser) as (app, gooeyApp):
                        widget = gooeyApp.configs[0].reifiedWidgets[0]
                        self.assertEqual(widget.getValue()['rawValue'], case.initialExpected)


if __name__ == '__main__':
    unittest.main()



import unittest
from argparse import ArgumentParser
from collections import namedtuple
from unittest.mock import patch
import wx
from gooey.tests import *

from gooey.tests.harness import instrumentGooey
from gooey import GooeyParser


class TestGooeyFilterableDropdown(unittest.TestCase):

    def make_parser(self, **kwargs):
        parser = GooeyParser(description='description')
        parser.add_argument('--dropdown', widget='FilterableDropdown', **kwargs)
        return parser

    def test_input_spawns_popup(self):
        parser = self.make_parser(choices=['alpha1', 'alpha2', 'beta', 'gamma'])
        with instrumentGooey(parser) as (app, gooeyApp):
            dropdown = gooeyApp.configs[0].reifiedWidgets[0]

            event = wx.CommandEvent(wx.wxEVT_TEXT, wx.Window.NewControlId())
            event.SetEventObject(dropdown.widget.GetTextCtrl())

            dropdown.widget.GetTextCtrl().ProcessEvent(event)
            self.assertTrue(
                dropdown.model.suggestionsVisible,
                dropdown.listbox.IsShown()
            )

    def test_arrow_key_selection_cycling(self):
        """
        Testing that the up/down arrow keys spawn the dropdown
        and cycle through its options wrapping around as needed.
        """
        Scenario = namedtuple('Scenario', [
            'key', 'expectVisible', 'expectedSelection', 'expectedDisplayValue'])

        choices = ['alpha', 'beta']
        # no text entered yet
        initial = Scenario(None, False, -1, '')
        scenarios = [
            # cycling down
            [
            Scenario(wx.WXK_DOWN, True, -1, ''),
            Scenario(wx.WXK_DOWN, True, 0, 'alpha'),
            Scenario(wx.WXK_DOWN, True, 1, 'beta'),
            # wraps around to top
            Scenario(wx.WXK_DOWN, True, 0, 'alpha')
        ],  # cycling up
            [
            Scenario(wx.WXK_UP, True, -1, ''),
            Scenario(wx.WXK_UP, True, 1, 'beta'),
            Scenario(wx.WXK_UP, True, 0, 'alpha'),
            # wraps around to top
            Scenario(wx.WXK_UP, True, 1, 'beta'),
        ]]

        for actions in scenarios:
            parser = self.make_parser(choices=choices)
            with instrumentGooey(parser) as (app, gooeyApp):
                dropdown = gooeyApp.configs[0].reifiedWidgets[0]
                # sanity check we're starting from our known initial state
                self.assertEqual(dropdown.model.suggestionsVisible, initial.expectVisible)
                self.assertEqual(dropdown.model.displayValue, initial.expectedDisplayValue)
                self.assertEqual(dropdown.model.selectedSuggestion, initial.expectedSelection)

                for action in actions:
                    self.pressButton(dropdown, action.key)
                    self.assertEqual(
                        dropdown.model.suggestionsVisible,
                        dropdown.listbox.IsShown()
                    )
                    self.assertEqual(
                        dropdown.model.displayValue,
                        action.expectedDisplayValue
                    )
                    self.assertEqual(
                        dropdown.model.selectedSuggestion,
                        action.expectedSelection
                    )


    def enterText(self, dropdown, text):
        event = wx.CommandEvent(wx.wxEVT_TEXT, wx.Window.NewControlId())
        event.SetString(text)
        dropdown.widget.GetTextCtrl().ProcessEvent(event)

    def pressButton(self, dropdown, keycode):
        event = mockKeyEvent(keycode)
        dropdown.onKeyboardControls(event)


def mockKeyEvent(keycode):
    """
    Manually bypassing the setters as they don'y allow
    the non wx.wxXXX event variants by default.
    The internal WX post/prcess machinery doesn't handle key
    codes well for some reason, thus has to be mocked and
    manually passed to the relevant handler.
    """
    event = wx.KeyEvent()
    event.KeyCode = keycode
    return event


if __name__ == '__main__':
    unittest.main()



import wx
import wx.richtext
import colored
import re

kColorList = ["#000000", "#800000", "#008000", "#808000", "#000080", "#800080", "#008080", "#c0c0c0",
    "#808080", "#ff0000", "#00ff00", "#ffff00", "#0000ff", "#ff00ff", "#00ffff", "#ffffff", "#000000",
    "#00005f", "#000087", "#0000af", "#0000d7", "#0000ff", "#005f00", "#005f5f", "#005f87", "#005faf",
    "#005fd7", "#005fff", "#008700", "#00875f", "#008787", "#0087af", "#0087d7", "#0087ff", "#00af00",
    "#00af5f", "#00af87", "#00afaf", "#00afd7", "#00afff", "#00d700", "#00d75f", "#00d787", "#00d7af",
    "#00d7d7", "#00d7ff", "#00ff00", "#00ff5f", "#00ff87", "#00ffaf", "#00ffd7", "#00ffff", "#5f0000",
    "#5f005f", "#5f0087", "#5f00af", "#5f00d7", "#5f00ff", "#5f5f00", "#5f5f5f", "#5f5f87", "#5f5faf",
    "#5f5fd7", "#5f5fff", "#5f8700", "#5f875f", "#5f8787", "#5f87af", "#5f87d7", "#5f87ff", "#5faf00",
    "#5faf5f", "#5faf87", "#5fafaf", "#5fafd7", "#5fafff", "#5fd700", "#5fd75f", "#5fd787", "#5fd7af",
    "#5fd7d7", "#5fd7ff", "#5fff00", "#5fff5f", "#5fff87", "#5fffaf", "#5fffd7", "#5fffff", "#870000",
    "#87005f", "#870087", "#8700af", "#8700d7", "#8700ff", "#875f00", "#875f5f", "#875f87", "#875faf",
    "#875fd7", "#875fff", "#878700", "#87875f", "#878787", "#8787af", "#8787d7", "#8787ff", "#87af00",
    "#87af5f", "#87af87", "#87afaf", "#87afd7", "#87afff", "#87d700", "#87d75f", "#87d787", "#87d7af",
    "#87d7d7", "#87d7ff", "#87ff00", "#87ff5f", "#87ff87", "#87ffaf", "#87ffd7", "#87ffff", "#af0000",
    "#af005f", "#af0087", "#af00af", "#af00d7", "#af00ff", "#af5f00", "#af5f5f", "#af5f87", "#af5faf",
    "#af5fd7", "#af5fff", "#af8700", "#af875f", "#af8787", "#af87af", "#af87d7", "#af87ff", "#afaf00",
    "#afaf5f", "#afaf87", "#afafaf", "#afafd7", "#afafff", "#afd700", "#afd75f", "#afd787", "#afd7af",
    "#afd7d7", "#afd7ff", "#afff00", "#afff5f", "#afff87", "#afffaf", "#afffd7", "#afffff", "#d70000",
    "#d7005f", "#d70087", "#d700af", "#d700d7", "#d700ff", "#d75f00", "#d75f5f", "#d75f87", "#d75faf",
    "#d75fd7", "#d75fff", "#d78700", "#d7875f", "#d78787", "#d787af", "#d787d7", "#d787ff", "#d7af00",
    "#d7af5f", "#d7af87", "#d7afaf", "#d7afd7", "#d7afff", "#d7d700", "#d7d75f", "#d7d787", "#d7d7af",
    "#d7d7d7", "#d7d7ff", "#d7ff00", "#d7ff5f", "#d7ff87", "#d7ffaf", "#d7ffd7", "#d7ffff", "#ff0000",
    "#ff005f", "#ff0087", "#ff00af", "#ff00d7", "#ff00ff", "#ff5f00", "#ff5f5f", "#ff5f87", "#ff5faf",
    "#ff5fd7", "#ff5fff", "#ff8700", "#ff875f", "#ff8787", "#ff87af", "#ff87d7", "#ff87ff", "#ffaf00",
    "#ffaf5f", "#ffaf87", "#ffafaf", "#ffafd7", "#ffafff", "#ffd700", "#ffd75f", "#ffd787", "#ffd7af",
    "#ffd7d7", "#ffd7ff", "#ffff00", "#ffff5f", "#ffff87", "#ffffaf", "#ffffd7", "#ffffff", "#080808",
    "#121212", "#1c1c1c", "#262626", "#303030", "#3a3a3a", "#444444", "#4e4e4e", "#585858", "#626262",
    "#6c6c6c", "#767676", "#808080", "#8a8a8a", "#949494", "#9e9e9e", "#a8a8a8", "#b2b2b2", "#bcbcbc",
    "#c6c6c6", "#d0d0d0", "#dadada", "#e4e4e4", "#eeeeee"]

class RichTextConsole(wx.richtext.RichTextCtrl):
    """
        An advanced rich test console panel supporting some Xterm control codes.
    """

    def __init__(self, parent):
        super(wx.richtext.RichTextCtrl, self).__init__(parent, -1, "", style=wx.richtext.RE_MULTILINE | wx.richtext.RE_READONLY)
        self.regex_urls=re.compile(r'\b((?:file://|https?://|mailto:)[^][\s<>|]*)')
        self.url_colour = wx.Colour(0,0,255)
        self.esc = colored.style.ESC
        self.end = colored.style.END
        self.noop = lambda *args, **kwargs: None

        self.actionsMap = {
            colored.style.BOLD: self.BeginBold,
            colored.style.RES_BOLD: self.EndBold,
            colored.style.UNDERLINED: self.BeginUnderline,
            colored.style.RES_UNDERLINED: self.EndUnderline,
            colored.style.RESET: self.EndAllStyles,
        }

        # Actions for coloring text
        for index, hex in enumerate(kColorList):
            escSeq = '{}{}{}'.format(colored.fore.ESC, index, colored.fore.END)
            wxcolor = wx.Colour(int(hex[1:3],16), int(hex[3:5],16), int(hex[5:],16), alpha=wx.ALPHA_OPAQUE)
            # NB : we use a default parameter to force the evaluation of the binding
            self.actionsMap[escSeq] = lambda bindedColor=wxcolor: self.BeginTextColour(bindedColor)
            
        self.Bind(wx.EVT_MOUSEWHEEL, self.onMouseWheel)


    def PreprocessAndWriteText(self, content):
        """Write text into console, while capturing URLs and making 
        them blue, underlined, and clickable.
        """
        textStream=iter(re.split(self.regex_urls, content))
        # The odd elements in textStream are plaintext;
        # the even elements are URLs.
        for plaintext in textStream:
            url=next(textStream, None)
            self.WriteText(plaintext)
            if url:    
                self.BeginTextColour(self.url_colour)
                self.BeginUnderline()
                self.BeginURL(url)
                self.WriteText(url)
                self.EndURL()
                self.EndUnderline()
                self.EndTextColour()
            
    def AppendText(self, content):
        """
        wx method overridden to capture the terminal control character and translate them into wx styles.
        Complexity : o(len(content))
        """
        self.SetInsertionPointEnd()
        unprocIndex = 0
        while True:
            # Invariant : unprocIndex is the starting index of the unprocessed part of the buffer
            escPos = content.find(self.esc, unprocIndex)
            if escPos == -1:
                break
            # Invariant : found an escape sequence starting at escPos
            # NB : we flush all the characters before the escape sequence, if any
            if content[unprocIndex:escPos]:
                self.PreprocessAndWriteText(content[unprocIndex:escPos])
            endEsc = content.find(self.end, escPos)
            if endEsc == -1:
                unprocIndex = escPos + len(self.esc)
                continue
            # Invariant : end of sequence has been found
            self.actionsMap.get(content[escPos:endEsc+1], self.noop)()
            unprocIndex = endEsc + 1
        # Invariant : unprocessed end of buffer is escape-free, ready to be printed
        self.PreprocessAndWriteText(content[unprocIndex:])
        self.ShowPosition(self.GetInsertionPoint())

    def onMouseWheel(self, event):
        if event.GetModifiers()==2 and event.GetWheelAxis()==wx.MOUSE_WHEEL_VERTICAL:
            if event.GetWheelRotation() >= event.GetWheelDelta():
                r=1.1
            elif event.GetWheelRotation() <= -event.GetWheelDelta():
                r=1.0/1.1
            else:
                return
            self.SetFontScale(self.GetFontScale() * r, True)
        else:
           event.Skip()



import unittest
from argparse import ArgumentParser

from python_bindings import constants
from python_bindings.config_generator import create_from_parser
from gooey.tests import *

class TextConfigGenerator(unittest.TestCase):

    def test_program_description(self):
        """
        Should use `program_description` if supplied, otherwise
        fallback to the description on the `parser`
        """

        parser = ArgumentParser(description="Parser Description")
        # when supplied explicitly, we assign it as the description
        buildspec = create_from_parser(parser, "", program_description='Custom Description')
        self.assertEqual(buildspec['program_description'], 'Custom Description')

        # when no explicit program_definition supplied, we fallback to the parser's description
        buildspec = create_from_parser(parser, "")
        self.assertEqual(buildspec['program_description'], 'Parser Description')

        # if no description is provided anywhere, we just set it to be an empty string.
        blank_parser = ArgumentParser()
        buildspec = create_from_parser(blank_parser, "")
        self.assertEqual(buildspec['program_description'], '')

    def test_valid_font_weights(self):
        """
        Asserting that only valid font-weights are allowable.
        """
        all_valid_weights = range(100, 1001, 100)
        for weight in all_valid_weights:
            parser = ArgumentParser(description="test parser")
            buildspec = create_from_parser(parser, "", terminal_font_weight=weight)
            self.assertEqual(buildspec['terminal_font_weight'], weight)

    def test_font_weight_defaults_to_normal(self):
        parser = ArgumentParser(description="test parser")
        # no font_weight explicitly provided
        buildspec = create_from_parser(parser, "")
        self.assertEqual(buildspec['terminal_font_weight'], constants.FONTWEIGHT_NORMAL)


    def test_invalid_font_weights_throw_error(self):
        parser = ArgumentParser(description="test parser")
        with self.assertRaises(ValueError):
            invalid_weight = 9123
            buildspec = create_from_parser(parser, "", terminal_font_weight=invalid_weight)


if __name__ == '__main__':
    unittest.main()


import time
import unittest

import wx
from gooey.gui.lang.i18n import _
from tests.integration.programs import all_widgets as all_widgets_module


class TestGooeyIntegration99(unittest.TestCase):


    def test_gooeyNormalRun(self):
        """ Tests the happy path through the default run mode of Gooey """
        from gooey.tests.integration import runner
        runner.run_integration(all_widgets_module, self.gooeySanityTest)


    def gooeySanityTest(self, app, buildSpec):
        time.sleep(1)
        try:
            # Check out header is present and showing data
            title = app.TopWindow.header._header.GetLabel()
            subtitle = app.TopWindow.header._subheader.GetLabel()
            self.assertEqual(title, buildSpec['program_name'])
            self.assertEqual(subtitle, buildSpec['program_description'])

            # switch to the run screen
            app.TopWindow.onStart()

            # Should find the expected test in the header
            title = app.TopWindow.header._header.GetLabel()
            subtitle = app.TopWindow.header._subheader.GetLabel()
            self.assertEqual(title,_("running_title"))
            self.assertEqual(subtitle, _('running_msg'))

            # Wait for Gooey to swap the header to the final screen
            while app.TopWindow.header._header.GetLabel() == _("running_title"):
                time.sleep(.1)

            # verify that we've landed on the success screen
            title = app.TopWindow.header._header.GetLabel()
            subtitle = app.TopWindow.header._subheader.GetLabel()
            self.assertEqual(title, _("finished_title"))
            self.assertEqual(subtitle, _('finished_msg'))

            # and that output was actually written to the console
            self.assertIn("Success", app.TopWindow.console.textbox.GetValue())
            time.sleep(1)
        except:
            app.TopWindow.Destroy()
            raise
        else:
            wx.CallAfter(app.TopWindow.Destroy)
            return None


if __name__ == '__main__':
    unittest.main()









import wx


def standard_layout(title, subtitle, widget):
    container = wx.BoxSizer(wx.VERTICAL)

    container.Add(title)
    container.AddSpacer(2)

    if subtitle:
        container.Add(subtitle, 1, wx.EXPAND)
        container.AddSpacer(2)
    else:
        container.AddStretchSpacer(1)

    container.Add(widget, 0, wx.EXPAND)
    return container



import wx

from gooey.gui import formatters
from gooey.gui.components.widgets.bases import TextContainer


class IntegerField(TextContainer):
    """
    An integer input field
    """
    widget_class = wx.SpinCtrl
    def getWidget(self, *args, **options):
        widget = self.widget_class(self,
                             value='',
                             min=self._options.get('min', 0),
                             max=self._options.get('max', 100))
        return widget

    def getWidgetValue(self):
        return self.widget.GetValue()

    def setValue(self, value):
        self.widget.SetValue(int(value))

    def formatOutput(self, metatdata, value):
        # casting to string so that the generic formatter
        # doesn't treat 0 as false/None
        return formatters.general(metatdata, str(value))


class DecimalField(IntegerField):
    """
    A decimal input field
    """
    widget_class = wx.SpinCtrlDouble

    def getWidget(self, *args, **options):
        widget = self.widget_class(self,
                             value='',
                             min=self._options.get('min', 0),
                             max=self._options.get('max', 100),
                             inc=self._options.get('increment', 0.01))
        widget.SetDigits(self._options.get('precision', widget.GetDigits()))
        return widget


    def setValue(self, value):
        self.widget.SetValue(value)





import re
from functools import reduce

import wx

from gooey.gui import formatters, events
from gooey.gui.util import wx_util
from gooey.util.functional import getin, ifPresent
from gooey.gui.validators import runValidator
from gooey.gui.components.util.wrapped_static_text import AutoWrappedStaticText
from gooey.gui.components.mouse import notifyMouseEvent


class BaseWidget(wx.Panel):
    widget_class = None

    def arrange(self, label, text):
        raise NotImplementedError

    def getWidget(self, parent, **options):
        return self.widget_class(parent, **options)

    def connectSignal(self):
        raise NotImplementedError

    def getSublayout(self, *args, **kwargs):
        raise NotImplementedError

    def setValue(self, value):
        raise NotImplementedError

    def setPlaceholder(self, value):
        raise NotImplementedError

    def receiveChange(self, *args, **kwargs):
        raise NotImplementedError

    def dispatchChange(self, value, **kwargs):
        raise NotImplementedError

    def formatOutput(self, metatdata, value):
        raise NotImplementedError


class TextContainer(BaseWidget):
    # TODO: fix this busted-ass inheritance hierarchy.
    # Cracking at the seems for more advanced widgets
    # problems:
    #   - all the usual textbook problems of inheritance
    #   - assumes there will only ever be ONE widget created
    #   - assumes those widgets are all created in `getWidget`
    #   - all the above make for extremely awkward lifecycle management
    #      - no clear point at which binding is correct.
    #   - I think the core problem here is that I couple the interface
    #     for shared presentation layout with the specification of
    #     a behavioral interface
    #     - This should be broken apart.
    #     - presentation can be ad-hoc or composed
    #     - behavioral just needs a typeclass of get/set/format for Gooey's purposes
    widget_class = None

    def __init__(self, parent, widgetInfo, *args, **kwargs):
        super(TextContainer, self).__init__(parent, *args, **kwargs)

        self.info = widgetInfo
        self._id = widgetInfo['id']
        self._meta = widgetInfo['data']
        self._options = widgetInfo['options']
        self.label = wx.StaticText(self, label=widgetInfo['data']['display_name'])
        self.help_text = AutoWrappedStaticText(self, label=widgetInfo['data']['help'] or '')
        self.error = AutoWrappedStaticText(self, label='')
        self.error.Hide()
        self.widget = self.getWidget(self)
        self.layout = self.arrange(*args, **kwargs)
        self.setColors()
        self.SetSizer(self.layout)
        self.bindMouseEvents()
        self.Bind(wx.EVT_SIZE, self.onSize)

        # 1.0.7 initial_value should supersede default when both are present
        if self._options.get('initial_value') is not None:
            self.setValue(self._options['initial_value'])
        # Checking for None instead of truthiness means False-evaluaded defaults can be used.
        elif self._meta['default'] is not None:
            self.setValue(self._meta['default'])

        if self._options.get('placeholder'):
            self.setPlaceholder(self._options.get('placeholder'))

        self.onComponentInitialized()

    def onComponentInitialized(self):
        pass

    def bindMouseEvents(self):
        """
        Send any LEFT DOWN mouse events to interested
        listeners via pubsub. see: gooey.gui.mouse for background.
        """
        self.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        self.label.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        self.help_text.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        self.error.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)
        self.widget.Bind(wx.EVT_LEFT_DOWN, notifyMouseEvent)

    def arrange(self, *args, **kwargs):
        wx_util.make_bold(self.label)
        wx_util.withColor(self.label, self._options['label_color'])
        wx_util.withColor(self.help_text, self._options['help_color'])
        wx_util.withColor(self.error, self._options['error_color'])

        self.help_text.SetMinSize((0,-1))

        layout = wx.BoxSizer(wx.VERTICAL)

        if self._options.get('show_label', True):
            layout.Add(self.label, 0, wx.EXPAND)
        else:
            self.label.Show(False)
            layout.AddStretchSpacer(1)

        layout.AddSpacer(2)
        if self.help_text and self._options.get('show_help', True):
            layout.Add(self.help_text, 1, wx.EXPAND)
            layout.AddSpacer(2)
        else:
            self.help_text.Show(False)
            layout.AddStretchSpacer(1)
        layout.Add(self.getSublayout(), 0, wx.EXPAND)
        layout.Add(self.error, 1, wx.EXPAND)

        self.error.Hide()
        return layout


    def setColors(self):
        wx_util.make_bold(self.label)
        wx_util.withColor(self.label, self._options['label_color'])
        wx_util.withColor(self.help_text, self._options['help_color'])
        wx_util.withColor(self.error, self._options['error_color'])
        if self._options.get('label_bg_color'):
            self.label.SetBackgroundColour(self._options.get('label_bg_color'))
        if self._options.get('help_bg_color'):
            self.help_text.SetBackgroundColour(self._options.get('help_bg_color'))
        if self._options.get('error_bg_color'):
            self.error.SetBackgroundColour(self._options.get('error_bg_color'))

    def getWidget(self, *args, **options):
        return self.widget_class(*args, **options)

    def getWidgetValue(self):
        raise NotImplementedError

    def getSublayout(self, *args, **kwargs):
        layout = wx.BoxSizer(wx.HORIZONTAL)
        layout.Add(self.widget, 1, wx.EXPAND)
        return layout

    def onSize(self, event):
        # print(self.GetSize())
        # self.error.Wrap(self.GetSize().width)
        # self.help_text.Wrap(500)
        # self.Layout()
        event.Skip()


    def getValue(self):
        regexFunc = lambda x: bool(re.match(userValidator, x))

        userValidator = getin(self._options, ['validator', 'test'], 'True')
        message = getin(self._options, ['validator', 'message'], '')
        testFunc = regexFunc \
                   if getin(self._options, ['validator', 'type'], None) == 'RegexValidator'\
                   else eval('lambda user_input: bool(%s)' % userValidator)
        satisfies = testFunc if self._meta['required'] else ifPresent(testFunc)
        value = self.getWidgetValue()

        return {
            'id': self._id,
            'cmd': self.formatOutput(self._meta, value),
            'rawValue': value,
            'test': runValidator(satisfies, value),
            'error': None if runValidator(satisfies, value) else message,
            'clitype': 'positional'
                        if self._meta['required'] and not self._meta['commands']
                        else 'optional'
        }

    def setValue(self, value):
        self.widget.SetValue(value)

    def setPlaceholder(self, value):
        if getattr(self.widget, 'SetHint', None):
            self.widget.SetHint(value)

    def setErrorString(self, message):
        self.error.SetLabel(message)
        self.error.Wrap(self.Size.width)
        self.Layout()

    def showErrorString(self, b):
        self.error.Wrap(self.Size.width)
        self.error.Show(b)

    def setOptions(self, values):
        return None

    def receiveChange(self, metatdata, value):
        raise NotImplementedError

    def dispatchChange(self, value, **kwargs):
        raise NotImplementedError

    def formatOutput(self, metadata, value):
        raise NotImplementedError




class BaseChooser(TextContainer):
    """ Base Class for the Chooser widget types """

    def setValue(self, value):
        self.widget.setValue(value)

    def setPlaceholder(self, value):
        self.widget.SetHint(value)

    def getWidgetValue(self):
        return self.widget.getValue()

    def formatOutput(self, metatdata, value):
        return formatters.general(metatdata, value)



import wx

from gooey.gui import formatters
from gooey.gui.components.widgets.bases import TextContainer
from gooey.gui.lang.i18n import _
from gooey.gui.util import wx_util


class CheckBox(TextContainer):

    widget_class = wx.CheckBox

    def arrange(self, *args, **kwargs):
        wx_util.make_bold(self.label)
        wx_util.withColor(self.label, self._options['label_color'])
        wx_util.withColor(self.help_text, self._options['help_color'])
        wx_util.withColor(self.error, self._options['error_color'])
        self.error.Hide()

        self.help_text.SetMinSize((0,-1))

        layout = wx.BoxSizer(wx.VERTICAL)
        if self._options.get('show_label', True):
            layout.Add(self.label, 0, wx.EXPAND)
        else:
            self.label.Show(False)
            layout.AddStretchSpacer(1)

        layout.AddSpacer(2)
        if self.help_text:
            hsizer = wx.BoxSizer(wx.HORIZONTAL)
            hsizer.Add(self.widget, 0)
            hsizer.Add(self.help_text, 1)
            layout.Add(hsizer, 1, wx.EXPAND)
            layout.AddSpacer(2)
        else:
            layout.Add(self.widget, 0, wx.EXPAND)
            layout.AddStretchSpacer(1)
        return layout


    def setValue(self, value):
        self.widget.SetValue(value)

    def getWidgetValue(self):
        return self.widget.GetValue()


    def formatOutput(self, metatdata, value):
        return formatters.checkbox(metatdata, value)


    def hideInput(self):
        self.widget.Hide()





class BlockCheckbox(CheckBox):
    """
    A block style layout which places the help text in the normal
    location rather than inline next to the checkbox. A replacement label
    called `block_label` is shown next to the checkbox control.

         +-----------------+
         |label            |
         |help_text        |
         |[ ] block_label  |
         +-----------------+
    This option tends to look better when there is a large amount of
    help text.
    """


    def arrange(self, *args, **kwargs):
        wx_util.make_bold(self.label)
        wx_util.withColor(self.label, self._options['label_color'])
        wx_util.withColor(self.help_text, self._options['help_color'])
        wx_util.withColor(self.error, self._options['error_color'])
        self.error.Hide()

        self.help_text.SetMinSize((0,-1))

        layout = wx.BoxSizer(wx.VERTICAL)

        if self._options.get('show_label', True):
            layout.Add(self.label, 0, wx.EXPAND)
        else:
            layout.AddStretchSpacer(1)

        layout.AddSpacer(2)
        if self.help_text and self._options.get('show_help', True):
            layout.Add(self.help_text, 1, wx.EXPAND)
            layout.AddSpacer(2)
        else:
            layout.AddStretchSpacer(1)

        layout.AddSpacer(2)

        block_label = self._options.get('checkbox_label', _('checkbox_label'))
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(self.widget, 0)
        hsizer.Add(wx.StaticText(self, label=block_label), 1)
        layout.Add(hsizer, 1, wx.EXPAND)
        layout.AddSpacer(2)

        return layout


import unittest
from unittest.mock import patch

from tests.harness import instrumentGooey
from gooey import GooeyParser
from gooey.tests import *

class TestGooeySlider(unittest.TestCase):

    def makeParser(self, **kwargs):
        parser = GooeyParser(description='description')
        parser.add_argument('--slider', widget="Slider", **kwargs)
        return parser


    def testSliderDefault(self):
        cases = [
            [{}, 0],
            [{'default': 0}, 0],
            [{'default': 10}, 10],
            [{'default': 76}, 76],
            # note that WX caps the value
            # unless explicitly widened via gooey_options
            [{'default': 81234}, 100],
            # here we set the max to something higher than
            # the default and all works as expected.
            # this is a TODO for validation
            [{'default': 81234, 'gooey_options': {'max': 99999}}, 81234],

            # Initial Value cases
            [{}, 0],
            [{'gooey_options': {'initial_value': 0}}, 0],
            [{'gooey_options': {'initial_value': 10}}, 10],
            [{'gooey_options': {'initial_value': 76}}, 76],
            # note that WX caps the value
            # unless explicitly widened via gooey_options
            [{'gooey_options': {'initial_value': 81234}}, 100],
            # here we set the max to something higher than
            # the default and all works as expected.
            # this is a TODO for validation
            [{'gooey_options': {'initial_value': 81234, 'max': 99999}}, 81234],
        ]
        for inputs, expected in cases:
            with self.subTest(inputs):
                parser = self.makeParser(**inputs)
                with instrumentGooey(parser) as (app, gooeyApp):
                    slider = gooeyApp.configs[0].reifiedWidgets[0]
                    self.assertEqual(slider.getValue()['rawValue'], expected)

    def testZerosAreReturned(self):
        """
        Originally the formatter was dropping '0' due to
        it being interpreted as falsey
        """
        parser = self.makeParser()
        with instrumentGooey(parser) as (app, gooeyApp):
            field = gooeyApp.configs[0].reifiedWidgets[0]
            result = field.getValue()
            self.assertEqual(result['rawValue'], 0)
            self.assertIsNotNone(result['cmd'])


if __name__ == '__main__':
    unittest.main()


from argparse import ArgumentParser, _SubParsersAction
from argparse import _MutuallyExclusiveGroup, _ArgumentGroup
from textwrap import dedent


class GooeySubParser(_SubParsersAction):
    def __init__(self, *args, **kwargs):
        super(GooeySubParser, self).__init__(*args, **kwargs)


# TODO: figure out how to correctly dispatch all of these
#       so that the individual wrappers aren't needed
class GooeyArgumentGroup(_ArgumentGroup):
    def __init__(self, parser, widgets, options, *args, **kwargs):
        self.parser = parser
        self.widgets = widgets
        self.options = options
        super(GooeyArgumentGroup, self).__init__(self.parser, *args, **kwargs)

    def add_argument(self, *args, **kwargs):
        widget = kwargs.pop('widget', None)
        metavar = kwargs.pop('metavar', None)
        options = kwargs.pop('gooey_options', None)
        action = super(GooeyArgumentGroup, self).add_argument(*args, **kwargs)
        self.parser._actions[-1].metavar = metavar
        self.widgets[self.parser._actions[-1].dest] = widget
        self.options[self.parser._actions[-1].dest] = options
        return action

    def add_argument_group(self, *args, **kwargs):
        options = kwargs.pop('gooey_options', {})
        group = GooeyArgumentGroup(self.parser, self.widgets, self.options, *args, **kwargs)
        group.gooey_options = options
        self._action_groups.append(group)
        return group

    def add_mutually_exclusive_group(self, *args, **kwargs):
        options = kwargs.pop('gooey_options', {})
        container = self
        group = GooeyMutuallyExclusiveGroup(container, self.parser, self.widgets, self.options, *args, **kwargs)
        group.gooey_options = options
        self.parser._mutually_exclusive_groups.append(group)
        return group


class GooeyMutuallyExclusiveGroup(_MutuallyExclusiveGroup):
    def __init__(self, container, parser, widgets, options, *args, **kwargs):
        self.parser = parser
        self.widgets = widgets
        self.options = options
        super(GooeyMutuallyExclusiveGroup, self).__init__(container, *args, **kwargs)

    def add_argument(self, *args, **kwargs):
        widget = kwargs.pop('widget', None)
        metavar = kwargs.pop('metavar', None)
        options = kwargs.pop('gooey_options', None)
        super(GooeyMutuallyExclusiveGroup, self).add_argument(*args, **kwargs)
        self.parser._actions[-1].metavar = metavar
        self.widgets[self.parser._actions[-1].dest] = widget
        self.options[self.parser._actions[-1].dest] = options



class GooeyParser(object):
    def __init__(self, **kwargs):
        self.__dict__['parser'] = ArgumentParser(**kwargs)
        self.widgets = {}
        self.options = {}
        if 'parents' in kwargs:
            for parent in kwargs['parents']:
                if isinstance(parent, self.__class__):
                    self.widgets.update(parent.widgets)
                    self.options.update(parent.options)

    @property
    def _mutually_exclusive_groups(self):
        return self.parser._mutually_exclusive_groups

    @property
    def _actions(self):
        return self.parser._actions

    @property
    def description(self):
        return self.parser.description

    def add_argument(self, *args, **kwargs):
        widget = kwargs.pop('widget', None)
        metavar = kwargs.pop('metavar', None)
        options = kwargs.pop('gooey_options', None)

        action = self.parser.add_argument(*args, **kwargs)
        self.parser._actions[-1].metavar = metavar

        action_dest = self.parser._actions[-1].dest
        if action_dest not in self.widgets or self.widgets[action_dest] is None:
            self.widgets[action_dest] = widget

        if action_dest not in self.options or self.options[action_dest] is None:
            self.options[self.parser._actions[-1].dest] = options

        self._validate_constraints(
            self.parser._actions[-1],
            widget,
            options or {},
            **kwargs
        )
        return action

    def add_mutually_exclusive_group(self, *args, **kwargs):
        options = kwargs.pop('gooey_options', {})
        group = GooeyMutuallyExclusiveGroup(self, self.parser, self.widgets, self.options, *args, **kwargs)
        group.gooey_options = options
        self.parser._mutually_exclusive_groups.append(group)
        return group

    def add_argument_group(self, *args, **kwargs):
        options = kwargs.pop('gooey_options', {})
        group = GooeyArgumentGroup(self.parser, self.widgets, self.options, *args, **kwargs)
        group.gooey_options = options
        self.parser._action_groups.append(group)
        return group

    def parse_args(self, args=None, namespace=None):
        return self.parser.parse_args(args, namespace)

    def add_subparsers(self, **kwargs):
        if self._subparsers is not None:
            self.error(_('cannot have multiple subparser arguments'))

        # add the parser class to the arguments if it's not present
        kwargs.setdefault('parser_class', type(self))

        if 'title' in kwargs or 'description' in kwargs:
            title = kwargs.pop('title', 'subcommands')
            description = kwargs.pop('description', None)
            self._subparsers = self.add_argument_group(title, description)
        else:
            self._subparsers = self._positionals

        # prog defaults to the usage message of this parser, skipping
        # optional arguments and with no "usage:" prefix
        if kwargs.get('prog') is None:
            formatter = self._get_formatter()
            positionals = self._get_positional_actions()
            groups = self._mutually_exclusive_groups
            formatter.add_usage(self.usage, positionals, groups, '')
            kwargs['prog'] = formatter.format_help().strip()

        # create the parsers action and add it to the positionals list
        parsers_class = self._pop_action_class(kwargs, 'parsers')
        action = parsers_class(option_strings=[], **kwargs)
        self._subparsers._add_action(action)

        # return the created parsers action
        return action

    def _validate_constraints(self, parser_action, widget, options, **kwargs):
        from gooey.python_bindings import constraints
        constraints.assert_listbox_constraints(widget, **kwargs)
        constraints.assert_visibility_requirements(parser_action, options)



    def __getattr__(self, item):
        return getattr(self.parser, item)

    def __setattr__(self, key, value):
        return setattr(self.parser, key, value)



"""Script for setuptools."""
import sys
from setuptools import setup, find_packages


with open('README.md') as readme:
    long_description = readme.read()

version = '1.0.8b5'

deps = [
    'Pillow>=4.3.0',
    'psutil>=5.4.2',
    'colored>=1.3.93',
    'pygtrie>=2.3.3'
]

if sys.version[0] == '3':
    deps.append('wxpython>=4.1.0')


setup(
    name='GooeyDev',
    version=version,
    url='http://pypi.python.org/pypi/Gooey/',
    author='Chris Kiehl',
    author_email='audionautic@gmail.com',
    description=('Turn (almost) any command line program into a full GUI '
                 'application with one line'),
    license='MIT',
    packages=find_packages(),
    install_requires=deps,
    include_package_data=True,
    dependency_links = ["http://www.wxpython.org/download.php"],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Desktop Environment',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Widget Sets',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License'
    ],
    long_description='''

Gooey (Beta)
############


Turn (almost) any Python Console Program into a GUI application with one line
-----------------------------------------------------------------------------

.. image:: https://cloud.githubusercontent.com/assets/1408720/7904381/f54f97f6-07c5-11e5-9bcb-c3c102920769.png


Quick Start
-----------

Gooey is attached to your code via a simple decorator on your `main` method.

.. code-block::

  from gooey import Gooey
  @Gooey      <--- all it takes! :)
  def main():
      # rest of code



With the decorator attached, run your program and the GUI will now appear!

Checkout the full documentation, instructions, and source on `Github <https://github.com/chriskiehl/Gooey>`_'''
)



import wx
from wx.lib.filebrowsebutton import FileBrowseButton
from typing import Dict
import os

class Submit(wx.Dialog):

    def __init__(self, *args, **kwargs):
        super(Submit, self).__init__(*args, **kwargs)


        self.submission_template = ""
        self.submission_cmd = ""
        self.submission_textfield : wx.TextCtrl = None
        self.filebrower: FileBrowseButton = None
        self.InitUI()

    def InitUI(self):

        panel = wx.Panel(self)

        hbox = wx.BoxSizer()
        sizer = wx.GridSizer(4, 1, 4, 4)

        #btn1 = wx.Button(panel, label='Info')
        #btn2 = wx.Button(panel, label='Error')

        row_submission_cmnd = wx.BoxSizer(wx.HORIZONTAL)
        lbl = wx.StaticText(panel, label=" Submission command: ")
        try:
            default_submission_cmd = os.environ['CRYOLO_SUBMIT_CMD']
        except:
            default_submission_cmd = 'sbatch'

        self.submission_textfield = wx.TextCtrl(panel, value=default_submission_cmd)
        row_submission_cmnd.Add(lbl, 0, flag = wx.EXPAND)
        row_submission_cmnd.Add(self.submission_textfield, 1, flag = wx.ALIGN_LEFT | wx.EXPAND)

        try:
            default_submission_script = os.environ['CRYOLO_SUBMIT_SCRIPT']
        except:
            default_submission_script = ''
        self.filebrower = FileBrowseButton(panel, labelText="Submission template:", initialValue=default_submission_script)  # wx.Button(panel, label='Alert')

        row_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        btn_ok = wx.Button(panel, label='OK')
        btn_cancel = wx.Button(panel, label='Cancel')
        row_ok_cancel.Add(btn_cancel)
        row_ok_cancel.Add(btn_ok)

        sizer.Add(self.filebrower, flag = wx.ALIGN_LEFT | wx.EXPAND)
        sizer.Add(row_submission_cmnd, flag = wx.ALIGN_LEFT | wx.EXPAND)
        sizer.AddStretchSpacer(1)
        sizer.Add(row_ok_cancel,1 , flag = wx.ALIGN_RIGHT)
        ### XXX_SXCMD_LINE_XXX
        hbox.Add(sizer, 1, wx.ALL, 15)
        panel.SetSizer(hbox)

        #btn1.Bind(wx.EVT_BUTTON, self.ShowMessage1)
        #btn2.Bind(wx.EVT_BUTTON, self.ShowMessage2)
        btn_cancel.Bind(wx.EVT_BUTTON, self.onCancel)
        btn_ok.Bind(wx.EVT_BUTTON, self.onOKPressed)

        self.SetSize((400, 200))
        self.SetTitle('Submit to queue')
        self.Centre()
        self.cancled = False

        #self.SetSize((500, 200))
        #self.SetTitle('Messages')
        #self.Centre()


    def onCancel(self, event):
        self.cancled=True
        self.Close()

    def onOKPressed(self, event):
        self.submission_cmd = self.submission_textfield.GetValue()
        self.submission_template = self.filebrower.GetValue()
        self.Close()
        '''
        dial = wx.MessageDialog(None, 'Unallowed operation', 'Exclamation',
            wx.OK | wx.ICON_EXCLAMATION)
        dial.ShowModal()
        '''
    def generate_submission_script(self,path_template : str, replacement_dict : Dict):
        import os
        filename = os.path.basename(path_template)
        filename = os.path.splitext(filename)[0]
        new_script=filename+"_job.sh"

        with open(path_template, "rt") as fin:
            with open(new_script, "wt") as fout:
                content = fin.read()
                for key, value in replacement_dict.items():
                    content = content.replace(key, value)
                fout.write(content)

        return new_script




import argparse
import os
import shlex
import unittest

from gooey.gui import formatters


class TestFormatters(unittest.TestCase):


    def test_counter_formatter(self):
        """
        Should return the first option repeated N times
        None if N is unspecified

        Issue #316 - using long-form argument caused formatter to produce incorrect output
        """
        expected_outputs = [
            (['-v', '--verbose'], '-v', 1),
            (['-v', '--verbose'], '-v -v', 2),
            (['-v', '--verbose'], '-v -v -v', 3),
            (['-v', '--verbose'], '', 0),
            # ensuring that log-forms are handled correctly
            (['--verbose', '-v'], '--verbose', 1),
            (['--verbose', '-v'], '--verbose --verbose', 2),
            (['--verbose', '-v'], '--verbose --verbose --verbose', 3),
            # single args
            (['-v'], '-v', 1),
            (['-v'], '-v -v', 2),
            (['--verbose'], '--verbose', 1),
            # bad inputs
            (['-v'], None, None),
            (['-v'], None, 'some-garbage'),
            (['-v'], None, 'af3gd'),
        ]

        for commands, expected, vebosity_level in expected_outputs:
            result = formatters.counter({'commands': commands}, vebosity_level)
            self.assertEqual(result, expected)
            # make sure that argparse actually accepts it as valid.
            if result:
                parser = argparse.ArgumentParser()
                parser.add_argument('-v', '--verbose', action='count')
                parser.parse_args(result.split())

    def test_multifilechooser_formatter(self):
        """
        Should return files (quoted), separated by spaces if there is more
        than one, preceeded by optional command if the argument is optional.

        Assumes the argument has been created with some form of nargs, which
        only makes sense for possibly choosing multiple values.
        """

        # Helper function to generalize the variants we need to test
        def multifilechooser_helper(names):
            # Note that the MultiFileChooser widget produces a single string with
            # paths separated by os.pathsep.
            if names:
                prefix = names[0] + ' '
            else:
                prefix = ''

            expected_outputs = [
                (names, None, ''),
                (names, prefix + '"abc"', 'abc'),
                (names, prefix + '"abc" "def"', os.pathsep.join(['abc', 'def'])),
                # paths with spaces
                (names, prefix + '"a b c"', 'a b c'),
                (names, prefix + '"a b c" "d e f"', os.pathsep.join(['a b c', 'd e f'])),
            ]

            for commands, expected, widget_result in expected_outputs:
                result = formatters.multiFileChooser({'commands': commands}, widget_result)
                self.assertEqual(result, expected)
                # make sure that argparse actually accepts it as valid.
                if result:
                    parser = argparse.ArgumentParser()
                    if not names:
                        names = ["file"]
                    parser.add_argument(names[0], nargs='+')
                    parser.parse_args(shlex.split(result))

        # Positional argument, with nargs
        multifilechooser_helper([])

        # Optional argument, with nargs
        multifilechooser_helper(["-f", "--file"])



import unittest

from gooey import PrefixTokenizers
from gui.components.filtering.prefix_filter import SearchOptions, PrefixSearch
from collections import namedtuple

TestData = namedtuple('TestData', [
    'options',
    'input_string',
    'expected_results',
])

Places = namedtuple('Places', [
    'kabul',
    'tirana',
    'kyoto',
    'tokyo'
])

class TestPrefixFilter(unittest.TestCase):


    def setUp(self):
        self.testdata = Places(
            'Afghanistan Kabul',
            'Albania Tirana',
            'Japan Kyoto',
            'Japan Tokyo'
        )

    def test_prefix_searching(self):
        p = self.testdata
        cases = [
            TestData({'ignore_case': True}, 'a', [p.kabul, p.tirana]),
            TestData({'ignore_case': True}, 'A', [p.kabul, p.tirana]),
            TestData({'ignore_case': False}, 'a', []),
            TestData({'ignore_case': False}, 'A', [p.kabul, p.tirana]),

            # when using the phrase tokenizer, the search input must
            # match starting from the beginning. So we find Afghanistan
            TestData({'choice_tokenizer': PrefixTokenizers.ENTIRE_PHRASE}, 'Afghan', [p.kabul]),
            # but we cannot look up Kyoto because the phrase begins with "Japan"
            TestData({'choice_tokenizer': PrefixTokenizers.ENTIRE_PHRASE}, 'Kyoto', []),
            # So if we start with "Japan K" it'll be returned
            TestData({'choice_tokenizer': PrefixTokenizers.ENTIRE_PHRASE}, 'Japan K', [p.kyoto]),



            # word tokenizer will split on all whitespace and index
            # each choice one for each UNIQUE word
            # so passing in 'a' will match "Af" and "Al" as usual
            TestData({'choice_tokenizer': PrefixTokenizers.WORDS}, 'a', [p.kabul, p.tirana]),
            # but now we can also find Kyoto without prefixing "japan" as we'd
            # need to do with the phrase tokenizer
            TestData({'choice_tokenizer': PrefixTokenizers.WORDS}, 'kyo', [p.kyoto]),

            # if we tokenize the input, we're perform two searches against the index
            # The default operator is AND, which means all the words in your search
            # input must match the choice for it to count as as a hit.
            # In this example, we index the choices under PHRASE, but set the input
            # tokenizer to WORDS. Our input 'Japan K' gets tokenized to ['Japan', 'K']
            # There is no phrase which starts with Both "Japan" and "K" so we get no
            # matches returned
            TestData({'choice_tokenizer': PrefixTokenizers.ENTIRE_PHRASE,
                      'input_tokenizer': PrefixTokenizers.WORDS}, 'Japan K', []),
            # Tokenize the choices by WORDS means we can now filter on both words
            TestData({'choice_tokenizer': PrefixTokenizers.WORDS,
                      'input_tokenizer': PrefixTokenizers.WORDS}, 'Jap K', [p.kyoto]),
            # the default AND behavior can be swapped to OR to facilitate matching across
            # different records in the index.
            TestData({'choice_tokenizer': PrefixTokenizers.WORDS,
                      'input_tokenizer': PrefixTokenizers.WORDS,
                      'operator': 'OR'}, 'Kyo Tok', [p.kyoto, p.tokyo]),

            # Turning on Suffix indexing allow matching anywhere within a word.
            # Now 'kyo' will match both the beginning 'Kyoto' and substring 'ToKYO'
            TestData({'choice_tokenizer': PrefixTokenizers.WORDS,
                      'input_tokenizer': PrefixTokenizers.WORDS,
                      'index_suffix': True}, 'kyo ', [p.kyoto, p.tokyo]),

            TestData({'choice_tokenizer': PrefixTokenizers.WORDS,
                      'input_tokenizer': PrefixTokenizers.WORDS,
                      'index_suffix': True}, 'j kyo ', [p.kyoto, p.tokyo]),
        ]

        for case in cases:
            with self.subTest(case):
                searcher = PrefixSearch(self.testdata, case.options)
                result = searcher.findMatches(case.input_string)
                self.assertEqual(result, case.expected_results)





'''
Utilities for loading, resizing and converting between PIL and WX image formats
'''

import six
from PIL import Image
import wx

from gooey.gui.three_to_four import bitmapFromBufferRGBA



def loadImage(img_path):
    return Image.open(img_path)


def resizeImage(im, targetHeight):
    im.thumbnail((six.MAXSIZE, targetHeight))
    return im


def wrapBitmap(im, parent):
    try:
        rgba = im.convert('RGBA').tobytes()
    except AttributeError:
        rgba = im.convert('RGBA').tostring()

    bitmapData = bitmapFromBufferRGBA(im, rgba)
    return wx.StaticBitmap(parent, bitmap=bitmapData)


if __name__ == '__main__':
    pass
